package jalview.hmmer;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertTrue;
import static org.testng.Assert.fail;

import jalview.bin.Jalview;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.HiddenMarkovModel;
import jalview.datamodel.SequenceI;
import jalview.gui.AlignFrame;
import jalview.gui.Desktop;
import jalview.ws.params.ArgumentI;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

public class HMMERTest {

  AlignFrame frame;

  @BeforeClass(alwaysRun = true)
  public void setUpBeforeClass() throws Exception
  {
    Jalview.main(
            new String[]
    { "-noquestionnaire", "-nonews", "-props",
                "test/jalview/hmmer/testProps.jvprops", "-open",
                "examples/uniref50.fa" });
    // FastaFile file = null;
    // file = new FastaFile(
    // new FileParse("examples/uniref50.fa", DataSourceType.FILE));
    // SequenceI[] seqs = file.getSeqsAsArray();
    // AlignmentI al = new Alignment(seqs);
    // frame = new AlignFrame(al, 150, 20);
    frame = Desktop.getAlignFrames()[0];
  }

  @AfterClass(alwaysRun = true)
  public static void tearDownAfterClass() throws Exception
  {
    Desktop.instance.closeAll_actionPerformed(null);
  }

  /**
   * Test with a dependency on locally installed hmmbuild binaries
   * 
   * @throws MalformedURLException
   * @throws IOException
   */
  @Test(groups = "External")
  public void testHMMBuildThenHMMAlign()
          throws MalformedURLException, IOException
  {
    /*
     * run hmmbuild - not the side-effect of selecting the HMM
     * sequence that gets added to the alignment
     */
    testHMMBuild();
    HiddenMarkovModel hmm = frame.getSelectedHMM();

    /*
     * now run hmmalign - with respect to the select HMM profile
     */
    testHMMAlign();
  }

  public void testHMMBuild()
  {
    HMMBuildThread thread = new HMMBuildThread(frame,
            new ArrayList<ArgumentI>());
    thread.hmmbuildWaitTillComplete();

    SequenceI seq = frame.getViewport().getAlignment().getSequenceAt(0);
    HiddenMarkovModel hmm = seq.getHMM();
    assertNotNull(hmm);

    assertEquals(hmm.getLength().intValue(), 148);
    assertEquals(hmm.getAlphabetType(), "amino");
    assertEquals(hmm.getName(), "Alignment");
    assertEquals(hmm.getEffectiveNumberOfSequences(), 0.648193, 0.0001);
    assertEquals(hmm.getConsensusAtAlignColumn(15), 's');
  }

  public void testHMMAlign()
  {
    HMMAlignThread thread = new HMMAlignThread(frame,
            new ArrayList<ArgumentI>());
    thread.hmmalignWaitTillComplete();

    AlignFrame[] alignFrames = Desktop.getAlignFrames();
    if (alignFrames == null)
    {
      fail("No align frame loaded");
    }

    /*
     * now have the original align frame, and another for realigned sequences
     */
    assertEquals(alignFrames.length, 2);
    AlignmentI original = alignFrames[0].getViewport().getAlignment();
    assertNotNull(original);
    AlignmentI realigned = alignFrames[1].getViewport().getAlignment();
    assertNotNull(realigned);
    assertNotNull(original.getHMMConsensusSequences());
    assertNotNull(realigned.getHMMConsensusSequences());

    SequenceI ferCapan = original.findName("FER_CAPAN");
    assertTrue(ferCapan.getSequenceAsString().startsWith("MA------SVSAT"));

    SequenceI ferCapanRealigned = realigned.findName("FER_CAPAN");
    assertTrue(ferCapanRealigned.getSequenceAsString()
            .startsWith("-------m-A----SVSAT"));
  }
}

